package java.lang.management;

import java.security.PrivilegedActionException;
import javax.management.NotCompliantMBeanException;
import javax.management.MBeanRegistrationException;
import javax.management.InstanceAlreadyExistsException;
import javax.management.StandardEmitterMBean;
import javax.management.StandardMBean;
import javax.management.NotificationEmitter;
import java.security.PrivilegedExceptionAction;
import java.util.HashSet;
import java.util.Set;
import java.util.Collections;
import java.io.IOException;
import javax.management.MalformedObjectNameException;
import javax.management.InstanceNotFoundException;
import javax.management.JMX;
import sun.misc.VM;
import java.security.AccessController;
import java.security.PrivilegedAction;
import javax.management.MBeanServerConnection;
import java.util.Iterator;
import sun.management.ExtendedPlatformComponent;
import javax.management.ObjectName;
import javax.management.DynamicMBean;
import java.util.Map;
import javax.management.MBeanServerFactory;
import javax.management.MBeanServerPermission;
import java.util.List;
import sun.management.ManagementFactoryHelper;
import javax.management.MBeanServer;

public class ManagementFactory
{
    public static final String CLASS_LOADING_MXBEAN_NAME = "java.lang:type=ClassLoading";
    public static final String COMPILATION_MXBEAN_NAME = "java.lang:type=Compilation";
    public static final String MEMORY_MXBEAN_NAME = "java.lang:type=Memory";
    public static final String OPERATING_SYSTEM_MXBEAN_NAME = "java.lang:type=OperatingSystem";
    public static final String RUNTIME_MXBEAN_NAME = "java.lang:type=Runtime";
    public static final String THREAD_MXBEAN_NAME = "java.lang:type=Threading";
    public static final String GARBAGE_COLLECTOR_MXBEAN_DOMAIN_TYPE = "java.lang:type=GarbageCollector";
    public static final String MEMORY_MANAGER_MXBEAN_DOMAIN_TYPE = "java.lang:type=MemoryManager";
    public static final String MEMORY_POOL_MXBEAN_DOMAIN_TYPE = "java.lang:type=MemoryPool";
    private static MBeanServer platformMBeanServer;
    private static final String NOTIF_EMITTER = "javax.management.NotificationEmitter";
    
    private ManagementFactory() {
    }
    
    public static ClassLoadingMXBean getClassLoadingMXBean() {
        return ManagementFactoryHelper.getClassLoadingMXBean();
    }
    
    public static MemoryMXBean getMemoryMXBean() {
        return ManagementFactoryHelper.getMemoryMXBean();
    }
    
    public static ThreadMXBean getThreadMXBean() {
        return ManagementFactoryHelper.getThreadMXBean();
    }
    
    public static RuntimeMXBean getRuntimeMXBean() {
        return ManagementFactoryHelper.getRuntimeMXBean();
    }
    
    public static CompilationMXBean getCompilationMXBean() {
        return ManagementFactoryHelper.getCompilationMXBean();
    }
    
    public static OperatingSystemMXBean getOperatingSystemMXBean() {
        return ManagementFactoryHelper.getOperatingSystemMXBean();
    }
    
    public static List<MemoryPoolMXBean> getMemoryPoolMXBeans() {
        return ManagementFactoryHelper.getMemoryPoolMXBeans();
    }
    
    public static List<MemoryManagerMXBean> getMemoryManagerMXBeans() {
        return ManagementFactoryHelper.getMemoryManagerMXBeans();
    }
    
    public static List<GarbageCollectorMXBean> getGarbageCollectorMXBeans() {
        return ManagementFactoryHelper.getGarbageCollectorMXBeans();
    }
    
    public static synchronized MBeanServer getPlatformMBeanServer() {
        SecurityManager securityManager = System.getSecurityManager();
        if (securityManager != null) {
            securityManager.checkPermission(new MBeanServerPermission("createMBeanServer"));
        }
        if (ManagementFactory.platformMBeanServer == null) {
            ManagementFactory.platformMBeanServer = MBeanServerFactory.createMBeanServer();
            for (PlatformComponent platformComponent : PlatformComponent.values()) {
                for (PlatformManagedObject platformManagedObject : platformComponent.getMXBeans(platformComponent.getMXBeanInterface())) {
                    if (!ManagementFactory.platformMBeanServer.isRegistered(platformManagedObject.getObjectName())) {
                        addMXBean(ManagementFactory.platformMBeanServer, platformManagedObject);
                    }
                    else {
                        continue;
                    }
                }
            }
            for (Map.Entry entry : ManagementFactoryHelper.getPlatformDynamicMBeans().entrySet()) {
                addDynamicMBean(ManagementFactory.platformMBeanServer, (DynamicMBean)entry.getValue(), (ObjectName)entry.getKey());
            }
            for (PlatformManagedObject platformManagedObject2 : ExtendedPlatformComponent.getMXBeans()) {
                if (!ManagementFactory.platformMBeanServer.isRegistered(platformManagedObject2.getObjectName())) {
                    addMXBean(ManagementFactory.platformMBeanServer, platformManagedObject2);
                }
                else {
                    continue;
                }
            }
        }
        return ManagementFactory.platformMBeanServer;
    }
    
    public static <T> T newPlatformMXBeanProxy(MBeanServerConnection connection, String str, final Class<T> clazz) throws IOException {
        if (!VM.isSystemDomainLoader((ClassLoader)AccessController.doPrivileged(new PrivilegedAction<ClassLoader>() {
            @Override
            public ClassLoader run() {
                return clazz.getClassLoader();
            }
        }))) {
            throw new IllegalArgumentException(str + " is not a platform MXBean");
        }
        try {
            ObjectName objectName = new ObjectName(str);
            if (!connection.isInstanceOf(objectName, clazz.getName())) {
                throw new IllegalArgumentException(str + " is not an instance of " + clazz);
            }
            return JMX.newMXBeanProxy(connection, objectName, clazz, connection.isInstanceOf(objectName, "javax.management.NotificationEmitter"));
        }
        catch (InstanceNotFoundException | MalformedObjectNameException cause) {
            throw new IllegalArgumentException(cause);
        }
    }
    
    public static <T extends PlatformManagedObject> T getPlatformMXBean(Class<T> clazz) {
        PlatformComponent platformComponent = PlatformComponent.getPlatformComponent(clazz);
        if (platformComponent == null) {
            PlatformManagedObject mxBean = ExtendedPlatformComponent.getMXBean(clazz);
            if (mxBean != null) {
                return mxBean;
            }
            throw new IllegalArgumentException(clazz.getName() + " is not a platform management interface");
        }
        else {
            if (!platformComponent.isSingleton()) {
                throw new IllegalArgumentException(clazz.getName() + " can have zero or more than one instances");
            }
            return platformComponent.getSingletonMXBean(clazz);
        }
    }
    
    public static <T extends PlatformManagedObject> List<T> getPlatformMXBeans(Class<T> clazz) {
        PlatformComponent platformComponent = PlatformComponent.getPlatformComponent(clazz);
        if (platformComponent == null) {
            PlatformManagedObject mxBean = ExtendedPlatformComponent.getMXBean(clazz);
            if (mxBean != null) {
                return Collections.singletonList(mxBean);
            }
            throw new IllegalArgumentException(clazz.getName() + " is not a platform management interface");
        }
        else {
            return Collections.unmodifiableList(platformComponent.getMXBeans(clazz));
        }
    }
    
    public static <T extends PlatformManagedObject> T getPlatformMXBean(MBeanServerConnection mBeanServerConnection, Class<T> clazz) throws IOException {
        PlatformComponent platformComponent = PlatformComponent.getPlatformComponent(clazz);
        if (platformComponent == null) {
            PlatformManagedObject mxBean = ExtendedPlatformComponent.getMXBean(clazz);
            if (mxBean != null) {
                return (T)newPlatformMXBeanProxy(mBeanServerConnection, mxBean.getObjectName().getCanonicalName(), clazz);
            }
            throw new IllegalArgumentException(clazz.getName() + " is not a platform management interface");
        }
        else {
            if (!platformComponent.isSingleton()) {
                throw new IllegalArgumentException(clazz.getName() + " can have zero or more than one instances");
            }
            return platformComponent.getSingletonMXBean(mBeanServerConnection, clazz);
        }
    }
    
    public static <T extends PlatformManagedObject> List<T> getPlatformMXBeans(MBeanServerConnection mBeanServerConnection, Class<T> clazz) throws IOException {
        PlatformComponent platformComponent = PlatformComponent.getPlatformComponent(clazz);
        if (platformComponent == null) {
            PlatformManagedObject mxBean = ExtendedPlatformComponent.getMXBean(clazz);
            if (mxBean != null) {
                return Collections.singletonList((T)newPlatformMXBeanProxy(mBeanServerConnection, mxBean.getObjectName().getCanonicalName(), clazz));
            }
            throw new IllegalArgumentException(clazz.getName() + " is not a platform management interface");
        }
        else {
            return Collections.unmodifiableList(platformComponent.getMXBeans(mBeanServerConnection, clazz));
        }
    }
    
    public static Set<Class<? extends PlatformManagedObject>> getPlatformManagementInterfaces() {
        HashSet<Class<? extends PlatformManagedObject>> s = new HashSet();
        PlatformComponent[] values = PlatformComponent.values();
        for (int length = values.length, i = 0; i < length; ++i) {
            s.add(values[i].getMXBeanInterface());
        }
        return Collections.unmodifiableSet(s);
    }
    
    private static void addMXBean(final MBeanServer mBeanServer, final PlatformManagedObject platformManagedObject) {
        try {
            AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                @Override
                public Void run() throws InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException {
                    DynamicMBean dynamicMBean;
                    if ((platformManagedObject instanceof DynamicMBean)) {
                        dynamicMBean = (DynamicMBean)DynamicMBean.class.cast(platformManagedObject);
                    }
                    else if ((platformManagedObject instanceof NotificationEmitter)) {
                        dynamicMBean = new StandardEmitterMBean(platformManagedObject, null, true, (NotificationEmitter)platformManagedObject);
                    }
                    else {
                        dynamicMBean = new StandardMBean(platformManagedObject, null, true);
                    }
                    mBeanServer.registerMBean(dynamicMBean, platformManagedObject.getObjectName());
                    return null;
                }
            });
        }
        catch (PrivilegedActionException ex) {
            throw new RuntimeException(ex.getException());
        }
    }
    
    private static void addDynamicMBean(final MBeanServer mBeanServer, final DynamicMBean dynamicMBean, final ObjectName objectName) {
        try {
            AccessController.doPrivileged(new PrivilegedExceptionAction<Void>() {
                @Override
                public Void run() throws InstanceAlreadyExistsException, MBeanRegistrationException, NotCompliantMBeanException {
                    mBeanServer.registerMBean(dynamicMBean, objectName);
                    return null;
                }
            });
        }
        catch (PrivilegedActionException ex) {
            throw new RuntimeException(ex.getException());
        }
    }
}
